--
--  ApertureEXIFExtractorAppDelegate.applescript
--  ApertureEXIFExtractor
--
--  Created by Robbie Duncan on 03/01/2010.
--  Copyright 2010 __MyCompanyName__. All rights reserved.
--

script ApertureEXIFExtractorAppDelegate
	property parent : class "NSObject"
	-- UI Elementes
	property projectProgressBar : missing value
	property projectProgressText : missing value
	property imageProgressBar : missing value
	property imageProgressText : missing value
	property startButton : missing value
	-- Aperture properties
	property projectList : missing value
	property projectCount : missing value
	property projectIndex : missing value
	property imageList : missing value
	property imageCount : missing value
	property imageIndex : missing value
	-- Stats store
	property statsStore : missing value
	-- Retry tracking
	property retry : 0
	property retryTagPos : 0
	
	on awakeFromNib()
		-- Reset the displays
		tell resultsText to setString_("Results will appear here")
	end awakeFromNib
	
	on applicationWillFinishLaunching_(aNotification)
	end applicationWillFinishLaunching_
	
	on applicationShouldTerminate_(sender)
		-- Insert code here to do any housekeeping before your application quits 
		return current application's NSTerminateNow
	end applicationShouldTerminate_
	
	on start_(sender)
		tell startButton to setEnabled_(false)
		tell application "Aperture"
			set projectList to projects
		end tell
		set projectCount to count of projectList
		set projectIndex to 1
		tell projectProgressBar to setMinValue_(0)
		tell projectProgressBar to setMaxValue_(projectCount)
		tell application "Aperture" to set imageList to image versions of item projectIndex of projectList
		set imageCount to count of imageList
		set imageIndex to 1
		tell imageProgressBar to setMinValue_(0)
		tell imageProgressBar to setMaxValue_(imageCount)
		tell class "NSMutableDictionary" of current application to set exifCounts to it's dictionary
		set imagesExamined to 0
		my performSelector_withObject_afterDelay_("loop", missing value, 1.0E-5)
	end start_
	
	-- This will be called over and over until we reach where we want to be (after start is clicked).  Note this this will be called with a delay to schedule it into the runloop and not block the UI
	on loop()
		try
			-- Update the UI values
			tell projectProgressText to setStringValue_("Project " & projectIndex & " of " & projectCount)
			tell projectProgressBar to setDoubleValue_(projectIndex as real)
			tell imageProgressText to setStringValue_("Image " & imageIndex & " of " & imageCount)
			tell imageProgressBar to setDoubleValue_(imageIndex as real)
			
			-- Extract the EXIF values
			set theItem to item imageIndex of imageList
			try
				tell application "Aperture" to set tagList to EXIF tags of theItem
			on error errText number errNum
				my performSelector_withObject_afterDelay_("loop", missing value, 5.0)
				return
			end try
			repeat with tagIndex from 1 to count of tagList
				if retryTagPos > 0 then
					set tagIndex to retryTagPos
					set retryTagPos to 0
				end if
				set theTag to item tagIndex of tagList
				try
					with timeout of 1 second
						tell application "Aperture" to set theName to name of theTag
					end timeout
				on error errText number errNum
					my performSelector_withObject_afterDelay_("loop", missing value, 1.0)
					return
				end try
				try
					with timeout of 1 second
						tell application "Aperture" to set theValue to value of theTag
					end timeout
				on error errText number errNum
					my performSelector_withObject_afterDelay_("loop", missing value, 1.0)
					return
				end try
				statsStore's incrementCountForTag_andValue_(theName as string, theValue as string)
				--set ret to my retry()
				if retry = 1 then
					set retry to 0
					set retryTagPos to 0
				end if
			end repeat
			
			-- Loop
			if (imageIndex < imageCount) then
				set imageIndex to imageIndex + 1
				my performSelector_withObject_afterDelay_("loop", missing value, 1.0E-6)
			else if (projectIndex < projectCount) then
				set projectIndex to projectIndex + 1
				tell application "Aperture" to set imageList to image versions of item projectIndex of projectList
				set imageCount to count of imageList
				set imageIndex to 1
				tell imageProgressBar to setMinValue_(0)
				tell imageProgressBar to setMaxValue_(imageCount)
				my performSelector_withObject_afterDelay_("loop", missing value, 1.0E-6)
			end if
			
		on error errText number errNum
			tell resultsText to setString_("ERROR in loop : " & errText)
		end try
	end loop
end script