//
//  ApertureStatsStore.m
//  ApertureEXIFExtractor
//
//  Created by Robbie Duncan on 06/01/2010.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "ApertureStatsStore.h"


@implementation ApertureStatsStore

-(id) init
{
	if ([super init])
	{
		tags = [[NSMutableDictionary alloc] init];
		orderedCountKeysByTag = [[NSMutableDictionary alloc] init];
	}
	tableWidth = 0;
	return self;
}

-(void) setOrderedTags
{
	NSMutableArray *tagNames = [NSMutableArray arrayWithArray:[tags allKeys]];
	[tagNames sortUsingSelector:@selector(compare:)];
	if (orderedTags != nil)
	{
		[orderedTags release];
	}
	orderedTags = [tagNames retain];
}

-(void) setOrderedCountKeysForTag:(NSString *) tag
{
	NSDictionary *counts = [tags objectForKey:tag];
	NSMutableArray *keys = [NSMutableArray arrayWithArray:[counts allKeys]];
	[keys sortUsingSelector:@selector(compare:)];
	[orderedCountKeysByTag setValue:keys forKey:tag];
}

-(void) incrementCountForTag:(NSString *) tag andValue:(NSString *) value
{
	NSMutableDictionary *counts = [tags objectForKey:tag];
	if (counts == nil)
	{
		counts = [NSMutableDictionary dictionary];
		[tags setObject:counts forKey:tag];
		[self setOrderedTags];
	}
	NSNumber *count = (NSNumber *) [counts objectForKey:value];
	if (count == nil)
	{
		[self setOrderedCountKeysForTag:tag];
		count = [NSNumber numberWithInt:0];
	}
	[counts setObject:[NSNumber numberWithInt:[count intValue]+1] forKey:value];
	int newWidth = [counts count];
	if (newWidth>tableWidth)
	{
		NSRect frame = [theTable frame];
		[theTable setFrame:frame];
		NSTableColumn *col = [[[NSTableColumn alloc] initWithIdentifier:[NSString stringWithFormat:@"%i",newWidth-1]] autorelease];
		[col setWidth:300.0];
		[theTable addTableColumn:col];
		tableWidth = newWidth;
	}
	[theTable reloadData];
}

- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [orderedTags count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{
	if ([[aTableColumn identifier] isEqualToString:@"header"])
		{
			return [orderedTags objectAtIndex:rowIndex];
		}
	else
	{
		int index = [[aTableColumn identifier] intValue];
		NSArray *orderedKeys = [orderedCountKeysByTag objectForKey:[orderedTags objectAtIndex:rowIndex]];
		if (index>=[orderedKeys count])
		{
			return @"";
		}
		NSString *key = [orderedKeys objectAtIndex:index];
		NSString *value = [[tags objectForKey:[orderedTags objectAtIndex:rowIndex]] objectForKey:key];
		return [NSString stringWithFormat:@"%@:%@",key,value];
	}	
}

-(void) redisplay
{
	[theTable reloadData];
}

-(IBAction) saveAs:(id) sender
{		
	int i, count, j, count2;
	NSSavePanel *sp = [NSSavePanel savePanel];
	[sp setAllowedFileTypes:[NSArray arrayWithObject:@"csv"]];
	[sp setAllowsOtherFileTypes:NO];
	if ([sp runModal]==NSFileHandlingPanelOKButton)
	{
		// Save the data!
		NSMutableString *output = [NSMutableString stringWithString:@"Aperture Stats Extractor gathered statistics\n\n"];
		count = [orderedTags count];
		for (i=0;i<count;i++)
		{
			[output appendFormat:@"EXIF Tag: %@\nValue,Count\n",[orderedTags objectAtIndex:i]];
			NSDictionary *counts = [tags objectForKey:[orderedTags objectAtIndex:i]];
			NSArray *keys = [orderedCountKeysByTag objectForKey:[orderedTags objectAtIndex:i]];
			count2 = [keys count];
			for (j=0;j<count2;j++)
			{
				[output appendFormat:@"%@,%@\n",[keys objectAtIndex:j],[counts objectForKey:[keys objectAtIndex:j]]];
			}
			[output appendString:@"\n"];
		}
		[output writeToURL:[sp URL] atomically:YES];
	}
}

@end
